create or replace
PACKAGE sbip_schedule AS
/*******************************************************************************
Author(s): Daniel McGhan (http://skillbuilders.com/apex, http://danielmcghan.us)
File: sbip_schedule_item.pks
Description:
   This package contains functions and procedures that either add functionality
   missing from DBMS_SCHEDULER or encapsulate existing logic to make calls easier.
   The package works with "schedule strings" generated by the SkillBuilders
   Schedule Item plug-in for APEX.
*******************************************************************************/
--<GLOBAL_PUBLIC_DATA_STRUCTURES>
   TYPE date_ntt IS TABLE OF DATE;
--</GLOBAL_PUBLIC_DATA_STRUCTURES>
/*******************************************************************************
This function returns the dates over which an event will occur. The function is
PIPELINED to allow for use in SQL.
1. schedule_string_in should be a "schedule string" generated by the SkillBuilders
   Schedule Item plug-in for APEX.
2. occurrence_limit_in should be used to limit the number of dates returned.
   Remember that events with no end date will recur infinitely.
3. window_start_in and window_end_in can be used to filter the dates returned
   so only those dates that occur between a certain "window" of time will be 
   returned. 
*******************************************************************************/
FUNCTION dates (
   schedule_string_in  IN VARCHAR2,
   occurrence_limit_in IN PLS_INTEGER := 1000,
   window_start_in     IN DATE := NULL,
   window_end_in       IN DATE := NULL
)
 
   RETURN DATE_NTT
      
   PIPELINED
   
   DETERMINISTIC;
/*******************************************************************************
This function returns the start date of an event. 
1. schedule_string_in should be a "schedule string" generated by the SkillBuilders
   Schedule Item plug-in for APEX.
*******************************************************************************/
FUNCTION start_date (
   schedule_string_in IN VARCHAR2
)
   
   RETURN DATE
   
   DETERMINISTIC;
/*******************************************************************************
This function returns the end date of an event. 
1. schedule_string_in should be a "schedule string" generated by the SkillBuilders
   Schedule Item plug-in for APEX.
*******************************************************************************/
FUNCTION end_date (
   schedule_string_in IN VARCHAR2
)
   
   RETURN DATE
   
   DETERMINISTIC;
/*******************************************************************************
This function returns the duration of an occurance of an event in seconds. 
1. schedule_string_in should be a "schedule string" generated by the SkillBuilders
   Schedule Item plug-in for APEX.
*******************************************************************************/
FUNCTION duration_seconds (
   schedule_string_in IN VARCHAR2
)
   
   RETURN PLS_INTEGER
   
   DETERMINISTIC;
/*******************************************************************************
This function returns a "summary" of an event's recurrence calendar. 
1. schedule_string_in should be a "schedule string" generated by the SkillBuilders
   Schedule Item plug-in for APEX.
*******************************************************************************/
FUNCTION calendar_summary (
   schedule_string_in IN VARCHAR2
)
   
   RETURN VARCHAR2
   
   DETERMINISTIC;
/******************************************************************************/
END sbip_schedule;
/