FUNCTION js_date_format(
   p_oracle_date_format IN VARCHAR2
)

   RETURN VARCHAR2

IS

   l_retval VARCHAR2(30);

BEGIN

   l_retval := UPPER(p_oracle_date_format);
   l_retval := REPLACE(l_retval, 'YYYY', 'yy');
   l_retval := REPLACE(l_retval, 'RRRR', 'yy');
   l_retval := REPLACE(l_retval, 'YY', 'y');
   l_retval := REPLACE(l_retval, 'RR', 'y');
   l_retval := REPLACE(l_retval, 'MONTH', 'MM');
   l_retval := REPLACE(l_retval, 'MON', 'M');
   l_retval := REPLACE(l_retval, 'MM', 'mm');
   l_retval := REPLACE(l_retval, 'DAY', 'DD');
   l_retval := REPLACE(l_retval, 'DY', 'D');
   l_retval := REPLACE(l_retval, 'DD', 'dd');
   
   RETURN l_retval;
   
END js_date_format;

FUNCTION render_sbip_schedule (
   p_item                IN APEX_PLUGIN.T_PAGE_ITEM,
   p_plugin              IN APEX_PLUGIN.T_PLUGIN,
   p_value               IN VARCHAR2,
   p_is_readonly         IN BOOLEAN,
   p_is_printer_friendly IN BOOLEAN 
)

   RETURN APEX_PLUGIN.T_PAGE_ITEM_RENDER_RESULT
   
IS

   l_retval         APEX_PLUGIN.T_PAGE_ITEM_RENDER_RESULT;
   l_name           VARCHAR2(30);
   l_date_format    VARCHAR2(30) := p_item.attribute_01;
   l_jqueryui_theme VARCHAR2(100) := p_plugin.attribute_02;
   l_inc_start_time CHAR(1) := p_item.attribute_03;
   l_inc_end_time   CHAR(1) := p_item.attribute_04;
   l_inc_all_day    CHAR(1) := p_item.attribute_05;
   l_inc_single     CHAR(1) := p_item.attribute_06;
   l_inc_daily      CHAR(1) := p_item.attribute_07;
   l_inc_weekly     CHAR(1) := p_item.attribute_08;
   l_inc_monthly    CHAR(1) := p_item.attribute_09;
   l_inc_yearly     CHAR(1) := p_item.attribute_10;
   l_onload_code    VARCHAR2(32767);
   l_crlf           CHAR(2) := CHR(13)||CHR(10);
   
BEGIN

   IF apex_application.g_debug 
   THEN
      apex_plugin_util.debug_page_item (
         p_plugin              => p_plugin,
         p_page_item           => p_item,
         p_value               => p_value,
         p_is_readonly         => p_is_readonly,
         p_is_printer_friendly => p_is_printer_friendly
      );
   END IF;

   l_name := apex_plugin.get_input_name_for_page_item(false);

   sys.htp.p(
         '<input type="hidden" name="' || l_name || '" id="' || p_item.name || '" '
      || 'value="' || p_value || '" ' || p_item.element_attributes || ' />'
      || '<div id="' || p_item.name || '_schedule" class="sbip_schedule"></div>'
   );

   apex_javascript.add_library (
      p_name      => 'jquery.ui.sbip_schedule',
      p_directory => p_plugin.file_prefix,
      p_version   => NULL
   );

   apex_javascript.add_library (
      p_name      => 'jquery.ui.button.min',
      p_directory => apex_application.g_image_prefix || 'libraries/jquery-ui/1.8/ui/minified/',
      p_version   => NULL
   );

   apex_javascript.add_library (
      p_name      => 'jquery.effects.pulsate.min',
      p_directory => apex_application.g_image_prefix || 'libraries/jquery-ui/1.8/ui/minified/',
      p_version   => NULL
   );
   
   apex_css.add_file (
      p_name      => 'jquery.ui.sbip_schedule',
      p_directory => p_plugin.file_prefix,
      p_version   => NULL
   );
   
   apex_css.add_file (
      p_name      => 'jquery-ui',
      p_directory => apex_application.g_image_prefix || 'libraries/jquery-ui/1.8/themes/' || l_jqueryui_theme || '/',
      p_version   => NULL
   );
   
   l_date_format := js_date_format(l_date_format);

   l_onload_code := 'apex.jQuery("div#' || p_item.name || '_schedule")'
      || '.sbip_schedule({' || l_crlf
      || '   scheduleItem:"' ||  p_item.name || '",' || l_crlf
      || '   incStartTime:"' || l_inc_start_time || '",' || l_crlf
      || '   incEndTime:"' || l_inc_end_time || '",' || l_crlf
      || '   incAllDay:"' || l_inc_all_day || '",' || l_crlf
      || '   dateFormat:"' || l_date_format || '",' || l_crlf
      || '   incSingle:"' || l_inc_single || '",' || l_crlf
      || '   incDaily:"' || l_inc_daily || '",' || l_crlf
      || '   incWeekly:"' || l_inc_weekly || '",' || l_crlf
      || '   incMonthly:"' || l_inc_monthly || '",' || l_crlf
      || '   incYearly:"' || l_inc_yearly || '"' || l_crlf
      || '});';
      
   apex_javascript.add_onload_code (
      p_code => l_onload_code
   );

   l_retval.is_navigable := true;
        
   RETURN l_retval;
    
END render_sbip_schedule;

FUNCTION is_valid_date(
   p_date_string IN VARCHAR2
)

   RETURN BOOLEAN

IS

   l_test_date DATE;
   
BEGIN

   l_test_date := TO_DATE(p_date_string, 'YYYYMMDDHHMIPM');

   RETURN TRUE;

EXCEPTION

   WHEN OTHERS
   THEN
      RETURN FALSE;
   
END is_valid_date;

FUNCTION validate_sbip_schedule(
   p_item   IN APEX_PLUGIN.T_PAGE_ITEM,
   p_plugin IN APEX_PLUGIN.T_PLUGIN,
   p_value  IN VARCHAR2
)

   RETURN APEX_PLUGIN.T_PAGE_ITEM_VALIDATION_RESULT

IS

   l_schedule_values  APEX_APPLICATION_GLOBAL.VC_ARR2;
   l_start_date_value VARCHAR2(20);
   l_end_date_value   VARCHAR2(20);
   l_duration_value   VARCHAR2(10);
   l_calendar_value   VARCHAR2(500);   
   l_retval           APEX_PLUGIN.T_PAGE_ITEM_VALIDATION_RESULT;

BEGIN

   l_schedule_values := apex_util.string_to_table(p_value, '|');
   
   IF l_schedule_values.exists(1)
   THEN
      l_start_date_value := l_schedule_values(1);

      IF NOT is_valid_date(l_start_date_value)
      THEN
         l_retval.message := 'Schedule start date is not valid';
         RETURN l_retval;
      END IF;
   ELSE
      l_retval.message := 'Schedule start date must have a value';
      RETURN l_retval;
   END IF;

   RETURN l_retval;
   
END validate_sbip_schedule;